from dataclasses import dataclass


@dataclass(frozen=True)
class JSONSchemaDialect:
    uri: str
    definitions_root_pointer: str
    all_refs: bool


@dataclass(frozen=True)
class JSONSchemaDraft202012Dialect(JSONSchemaDialect):
    uri: str = "https://json-schema.org/draft/2020-12/schema"
    definitions_root_pointer: str = "#/defs"
    all_refs: bool = False


@dataclass(frozen=True)
class OpenAPISchema31Dialect(JSONSchemaDialect):
    uri: str = "https://spec.openapis.org/oas/3.1/dialect/base"
    definitions_root_pointer: str = "#/components/schemas"
    all_refs: bool = True


DRAFT_2020_12 = JSONSchemaDraft202012Dialect()
OPEN_API_3_1 = OpenAPISchema31Dialect()


__all__ = ["JSONSchemaDialect", "DRAFT_2020_12", "OPEN_API_3_1"]



import enum
import sys

__all__ = [
    "PY_37",
    "PY_38",
    "PY_39",
    "PY_310",
    "PY_37_MIN",
    "PY_38_MIN",
    "PY_39_MIN",
    "PY_310_MIN",
    "PY_311_MIN",
    "PEP_585_COMPATIBLE",
    "PEP_586_COMPATIBLE",
    "Sentinel",
]


PY_37 = sys.version_info.major == 3 and sys.version_info.minor == 7
PY_38 = sys.version_info.major == 3 and sys.version_info.minor == 8
PY_39 = sys.version_info.major == 3 and sys.version_info.minor == 9
PY_310 = sys.version_info.major == 3 and sys.version_info.minor == 10
PY_311 = sys.version_info.major == 3 and sys.version_info.minor == 11
PY_312_MIN = sys.version_info.major == 3 and sys.version_info.minor >= 12

PY_311_MIN = PY_311 or PY_312_MIN
PY_310_MIN = PY_310 or PY_311_MIN
PY_39_MIN = PY_39 or PY_310_MIN
PY_38_MIN = PY_38 or PY_39_MIN
PY_37_MIN = PY_37 or PY_38_MIN

PEP_585_COMPATIBLE = PY_39_MIN  # Type Hinting Generics In Standard Collections
PEP_586_COMPATIBLE = PY_38_MIN  # Literal Types


class Sentinel(enum.Enum):
    MISSING = enum.auto()



from dataclasses import dataclass
from numbers import Number
from typing import Dict, Set

from mashumaro.jsonschema.models import JSONSchema


class Annotation:
    pass


class Constraint(Annotation):
    pass


class NumberConstraint(Constraint):
    pass


@dataclass(unsafe_hash=True)
class Minimum(NumberConstraint):
    value: Number


@dataclass(unsafe_hash=True)
class Maximum(NumberConstraint):
    value: Number


@dataclass(unsafe_hash=True)
class ExclusiveMinimum(NumberConstraint):
    value: Number


@dataclass(unsafe_hash=True)
class ExclusiveMaximum(NumberConstraint):
    value: Number


@dataclass(unsafe_hash=True)
class MultipleOf(NumberConstraint):
    value: Number


class StringConstraint(Constraint):
    pass


@dataclass(unsafe_hash=True)
class MinLength(StringConstraint):
    value: int


@dataclass(unsafe_hash=True)
class MaxLength(StringConstraint):
    value: int


@dataclass(unsafe_hash=True)
class Pattern(StringConstraint):
    value: str


class ArrayConstraint(Constraint):
    pass


@dataclass(unsafe_hash=True)
class MinItems(ArrayConstraint):
    value: int


@dataclass(unsafe_hash=True)
class MaxItems(ArrayConstraint):
    value: int


@dataclass(unsafe_hash=True)
class UniqueItems(ArrayConstraint):
    value: bool


@dataclass(unsafe_hash=True)
class Contains(ArrayConstraint):
    value: JSONSchema


@dataclass(unsafe_hash=True)
class MinContains(ArrayConstraint):
    value: int


@dataclass(unsafe_hash=True)
class MaxContains(ArrayConstraint):
    value: int


class ObjectConstraint(Constraint):
    pass


@dataclass(unsafe_hash=True)
class MaxProperties(ObjectConstraint):
    value: int


@dataclass(unsafe_hash=True)
class MinProperties(ObjectConstraint):
    value: int


@dataclass
class DependentRequired(ObjectConstraint):
    value: Dict[str, Set[str]]


__all__ = [
    "Annotation",
    "MultipleOf",
    "Maximum",
    "ExclusiveMaximum",
    "Minimum",
    "ExclusiveMinimum",
    "MaxLength",
    "MinLength",
    "Pattern",
    "MaxItems",
    "MinItems",
    "UniqueItems",
    "Contains",
    "MaxContains",
    "MinContains",
    "MaxProperties",
    "MinProperties",
    "DependentRequired",
]
